/*
    $File$
    Date        Author          Changes
    Nov 21 2000 Richard Jones   Created

    Copyright (c) 2000, Richard Jones
    All rights reserved.
 */
 
package MetaTF.Parser;

import java.util.*;
import MetaTF.Panic;

/**
 * Tag field
 *
 * @see <a href="metatf.jjt">The MetaTF Grammar</a>
 * @see MetaTF.Parser.Field
 * @version rel-1-2 ($Revision: 1.3 $, $Date: 2001/03/29 15:53:18 $),
 * @author Richard Jones
**/
public class TagField extends Field {
  private static final String cvsheader = "$Header: /home/cur/rej/cvs/MetaTF/src/MetaTF/Parser/TagField.java,v 1.3 2001/03/29 15:53:18 rej Exp $";

    private static final int NOTAG = -1;

    /** The value of the tag **/
    private int value = NOTAG;

    /**
     * Allocate a new object representing the tag
     *
     * @param v the tag's value
     * @param l the line in the DTD at which this tag was found
    **/
    TagField(int v, int l) {
        value = v;
        beginLine = l;
    }
    
    /**
     * Allocate a new object representing the tag
     *
     * @param l the line in the DTD at which this tag was found
    **/
    TagField(int l) {
        beginLine = l;
    }

    public String getName() { return "tag";}

    public int getTag() throws NoTagException { return value; }

    public boolean isTagged() { return true; }

    /**
     * Set the value of the tag
     *
     * @param i the value
     * @exception DTDException IF i<0
    **/
    void setTag(int i) throws DTDException {
        if(i>=0)
            value = i;
        else
            error("tag values must be non-negative integers (found "+i+")");
    }

    /**
     * Set the value of the tag
     *
     * @param v the value
     * @exception DTDException IF v==null || v is not a NumberValue || the value is negative
     * @see MetaTF.Parser.TagField#setTag
    **/
    public void setTag(Value v) throws DTDException {
        if(v==null)
            Panic.panic("l. "+beginLine+": TagField.setTag gets null");
        else if(!(v instanceof NumberValue))
            error("tag values must be non-negative integers (found "+v+")");            
        else 
            setTag(((NumberValue)v).getNumber());
    }

    
    Field replaceFields(Map inheritedRecords0, Map inheritedRecords, Map inheritedFields) {
    	return this;
    }

    void checkFields() throws DTDException {
        if(value==NOTAG) 
            error("tag found without a value");
    }

    public boolean containsField(String s) {
        return s.equals("tag");
    }

    /**
     * Add a property to this field.
     * Only valid values are:
     * <PRE>
     *     type = int or null
     *     field = tag
     *     property.value = "". NumberValue>=0        (1)
     *                    | value . NumberValue>=0    (1)
     *                    | encoding . None     (2)
     * </PRE>
     *
     * @param type the type
     * @param field the name of the field. The property is only added if the field names match
     * @param property the property
     * @param value the value of the property
     * @exception DTDException IF property is not well-formed
    **/
    boolean addProperty(String type, String field, String property, Value value, int l) 
            throws DTDException {

        if(!field.equals("tag"))     // nothing to do with tag field
            return false;

        //Deal with type
        if(!(type==null ||
             type.equals("int") ||
             type.equals("")
            ))
            error(l, " cannot give tag a type other than int: "+type);
        // else Tags are always int so do nothing 

        // Deal with property
        if(property==null || property.equals("")) 
            property = "value";
        if(property.equals("value")) {
                setTag(value);      // (1)
        }
        else if(property.equals("encoding")) {
            if(value==null) {
                Panic.panic("l. "+l+": TagField.addPropery got .encoding but no value");
            }
            if(!(value instanceof None))
                error(l, "cannot give a tag an encoding other than None "+
                      "(found "+value+")");
            // else                    (2) Tags always have encoding none 
        }
        else {
             error(l, "cannot give tag a property other than .value or .encoding "+
                   "(found "+property+")");
        }
        
        return true;
    }

    /**
     * Add a subfield. This is always an error.
     * Only public because it is an interface method.
    **/
    public void addField(Field f) { 
        Panic.panic("l. "+beginLine+": cannot addField to a TagField ("+f+")");
    }

    public int getNumNamedFields() { return 1; }

    public String getType() { return "int"; }
    
    /**
     * Report an error
     *
     * @param l the line in the DTD where the error was found
     * @param s an error message
     * @exception DTDException  ALWAYS
    **/
    private void error(int l, String s) throws DTDException {
        throw new DTDException("l. "+l+": "+s);
    }
    
    /**
     * Report an error at the line in the DTD where this tag was found
     *
     * @param s an error message
     * @exception DTDException  ALWAYS
    **/
    private void error(String s) throws DTDException {
        error(beginLine, s);
    }
    
    public String dump(String prefix) {
        String result = prefix+"tag.value=";
        if(value==NOTAG)
            return result+"not set";
        else
            return result+value;
    }

    public Object accept(DTDVisitor visitor, Object data) {
        return visitor.visit(this, data);
    }
}
